package oliot.aika.rajapinta;

/**
 * Luokka toteuttamaan sovitun julkisen rajapinnan
 * @author  Vesa Lappalainen @version 1.0, 01.02.2003
 * @author Santtu Viitanen @version 1.1, 8.7.2011
 * @example
 * <pre name="test">
 * Aika a1 = new Aika();
 * Aika a2 = new Aika(13);
 * Aika a3 = new Aika(14,25);
 * a1.toString() === "00:00"; a2.toString() === "13:00";  
 * a3.toString() === "14:25";
 * a1.aseta(12,15); a2.aseta(16,-15);
 * a1.toString() === "12:15"; a2.toString() === "15:45";
 * Aika.lisaa(a1,55); a1.toString() === "13:10";
 * a1.getH() === 13; a1.getM() === 10;
 * </pre>
 */
public class Aika implements AikaRajapinta, Cloneable {

    private int h, m;


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param ih asetettavat tunnit
     * @param im asetettavat minuutit
     * @return yli- tai alijneet vuorokaudet
     * @example
     * <pre name="test">
     *   Aika a = new Aika();
     *   a.aseta(25,12)  ===  1;  a.toString() === "01:12";
     *   a.aseta(3,90)   ===  0;  a.toString() === "04:30";
     *   a.aseta(3,-10)  ===  0;  a.toString() === "02:50";
     *   a.aseta(-3,-10) === -1;  a.toString() === "20:50";
     * </pre>
     */
    @Override
    public final int aseta(int ih, int im) {
        int th = ih;
        int tm = im;
        th += tm / 60; // liiat minuutit tunteihin
        tm %= 60; // minuutit ville -59 - 59
        int vrk = th / 24; // liiat tunnit vuorokausiin
        th %= 24; // tunnit vlille 0-23

        if (tm < 0) {
            tm += 60;
            th--;
        } //negatiiviset arvot
        if (th < 0) {
            th += 24;
            vrk--;
        }

        this.h = th; // asetetaan lasketut arvot attribuutteeihin
        this.m = tm;
        return vrk; // motako vuorokautta ji yli tai ali
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param ih asetettavat tunnit
     * @return yli- tai alijneet vuorokaudet
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,30);
     *   a.aseta(25)  ===  1;  a.toString() === "01:00";
     *   a.aseta(3)   ===  0;  a.toString() === "03:00";
     *   a.aseta(-3)  === -1;  a.toString() === "21:00";
     * </pre>
     */
    @Override
    public final int aseta(int ih) {
        return aseta(ih,0);
    }
    
    
    /**
     * Alustetaan aika 0:00
     * @example
     * <pre name="test">
     *   Aika a = new Aika();
     *   a.toString() === "00:00";
     * </pre>
     */
    public Aika() {
        this.h = 0;
        this.m = 0;
    }


    /**
     * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
     * @param h asetettavat tunnit
     * @example
     * <pre name="test">
     *   Aika a = new Aika(5);
     *   a.toString() === "05:00";
     * </pre>
     */
    public Aika(int h) {
        aseta(h, 0);
    }


    /**
     * Alustaa ajan
     * @param h tunnit
     * @param m minuutit
     * Aika a = new Aika(12,30);
     * a.toString() === "12:30";
     */
    public Aika(int h, int m) { // Muodostaja
        aseta(h, m);
    }


    /**
     * @return aika merkkijonona muodossa 12:05
     * @example
     * <pre name="test">
     * Aika a = new Aika(12,5);
     * a.toString() === "12:05";
     * </pre>
     */
    @Override
    public String toString() {
        return String.format("%02d:%02d", getH(), getM());
    }


    /**
     * Lis aikaan valitun minuuttimrn
     * @param lisaMin listtv minuuttimr
     * @return ylimenneet vuorokaudet
     * @example
     * <pre name="test">
     * Aika a = new Aika(12,50);
     * a.lisaa(30) === 0; a.toString() === "13:20";
     * a.lisaa(12*60) === 1; a.toString() === "01:20";
     * </pre>
     */
    @Override
    public int lisaa(int lisaMin) {
        return aseta(h, m + lisaMin);
    }


    /**
     * Lis aaikaan minuutit
     * @param aika johon listn
     * @param lisa_min paljonko listn
     * @return ylimenneet vuorokaudet
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,50);
     *   lisaa(a,30) === 0; a.toString() === "13:20";
     * </pre>
     */
    public static int lisaa(Aika aika, int lisa_min) {
        return aika.lisaa(lisa_min);
    }

    
    /**
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,50);
     *   a.getH() === 12;
     * </pre>
     */
    @Override
    public int getH() {
        return h;
    }


    /**
     * @example
     * <pre name="test">
     *   Aika a = new Aika(12,50);
     *   a.getM() === 50;
     * </pre>
     */
    @Override
    public int getM() {
        return m;
    }
    
    /**
     * @example
     * <pre name="test">
     * Aika a1 = new Aika(13,37);
     * Aika a2 = new Aika(13,37);
     * a1 === a2
     * </pre>
     */
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof AikaRajapinta))
            return false;
        AikaRajapinta a = (AikaRajapinta) o;
        return getH() == a.getH() && getM() == a.getM();
    }

    /**
     * @example
     * <pre name="test">
     * Aika aika = new Aika(13,37);
     * Aika aika2 = aika.clone();
     * aika2.toString() === "13:37";
     * </pre>
     */
    @Override
    public Aika clone() {
        Aika kopio = null;
        try {
            kopio = (Aika)super.clone();
        } catch (CloneNotSupportedException e) {
            // peritn Objectista ja se tukee kloonausta
        }
        return kopio;
    }

    /**
     * Aika sekunteina vuorokauden alusta
     * @example
     * <pre name="test">
     * new Aika(13,37).hashCode() === 49020;
     * </pre>
     */
    @Override
    public int hashCode() { return 3600 * getH() + 60 * getM(); }

}
